/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext,Nexus,Sonatype,NX*/
//noinspection JSUnusedGlobalSymbols
NX.define('Nexus.ssl.configuration.GlobalConfigurationView', {
  extend : 'Ext.Panel',
  requires : ['Nexus.ssl.configuration.CertificateManagerGrid',
              'Nexus.ssl.certificate.CertificateDetailsView',
              'Nexus.ssl.Icons'],

  statics : {
    tabId : 'nexus-ssl-global-config'
  },

  /**
   * Show details view or the 'select a cert' message
   * @param id the record id for the trusted key to show
   */
  showDetails : function(id) {
    var self = this, grid, record,
        icons = Nexus.ssl.Icons;

    self.detailsView.removeAll();

    if (!id) {
      self.detailsView.add(Nexus.ssl.certificate.CertificateDetailsView.NO_DETAIL);
      self.detailsView.setTitle('No Selection', icons.get('warning').cls);
      self.doLayout();
      return;
    }

    grid = self.findByType('certificate-manager')[0];
    record = grid.store.query('id', id).get(0);

    if (record === undefined) {
      self.detailsView.add(Nexus.ssl.certificate.CertificateDetailsView.NO_DETAIL);
      self.detailsView.setTitle('No Selection', icons.get('warning').cls);
    } else {
      self.detailsView.add({
        xtype : 'certificate-details',
        autoDestroy : true,
        certificate : record.data
      });
      self.detailsView.setTitle(record.data.subjectCommonName, icons.get('certificate').cls);
    }

    self.doLayout();
  },

  initComponent : function() {
    var self = this;

    self.layout = 'border';

    self.detailsView = NX.create('Ext.Panel', {
      title : 'No Selection',
      iconCls: Nexus.ssl.Icons.get('warning').cls,
      region : 'south',
      layout : 'fit',
      split : true,
      border : true,
      items : [Nexus.ssl.certificate.CertificateDetailsView.NO_DETAIL],
      listeners : {
        /**
         * set details view to be 2/3 of the available height initially
         */
        afterrender : {
          fn : function(cmp) {
            cmp.setHeight(self.getHeight() / 1.5);
          },
          single : true
        }
      }
    });

    self.certificateManager = NX.create('Nexus.ssl.configuration.CertificateManagerGrid', {
      xtype : 'certificate-manager',
      region : 'center',
      listeners : {
        certsRemoved : function() {
          self.showDetails(null);
        },
        certSelected : function(record) {
          self.bookmark = record.id;
          Sonatype.utils.updateHistory(self);
          self.showDetails(record.id);
        },
        certDeselected : function(record) {
          if ( self.bookmark === record.id ) {
            self.showDetails(null);
          }
        }
      }
    });

    self.items = [
      self.certificateManager,
      self.detailsView
    ];

    self.constructor.superclass.initComponent.apply(self, arguments);
  },

  getBookmark : function() {
    return this.bookmark;
  },

  applyBookmark : function(bookmark) {
    var self = this, grid;

    self.bookmark = bookmark;
    grid = self.findByType('certificate-manager')[0];
    grid.select(bookmark);
    self.showDetails(bookmark);
  }


}, function() {
  Sonatype.Events.addListener('nexusNavigationInit', function(panel) {
    var sp = Sonatype.lib.Permissions;
    if (sp.checkPermission('nexus:ssl:truststore', sp.READ)) {
      panel.add({
        title : 'SSL Certificates',
        tabId : Nexus.ssl.configuration.GlobalConfigurationView.tabId,
        tabCode : Nexus.ssl.configuration.GlobalConfigurationView,
        sectionId : 'st-nexus-config'
      });
    }
  });
});
