/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging repository data store.
 *
 * @since 2.3
 *
 * @see java:com.sonatype.nexus.staging.api.dto.stagingProfileRepository
 */
NX.define('Nexus.staging.repository.RepositoryStore', {
    extend: 'Ext.data.JsonStore',

    constructor: function (config) {
        var self = this,
            config = config || {},
            ST = Ext.data.SortTypes;

        Ext.apply(config, {
            url: Sonatype.config.servicePath + '/staging/profile_repositories',
            root: 'data',
            id: 'repositoryId',

            // NOTE: There is more data returned from the REST resource, but it is not consumed by the UI

            fields: [
                { name: 'repositoryId' },
                { name: 'repositoryURI' },

                { name: 'profileId' },
                { name: 'profileName' },
                { name: 'profileType' }, // 'group' or 'repository'

                { name: 'policy' }, // always 'release' ???
                { name: 'provider' }, // maven1, maven2, etc.
                { name: 'description' },
                { name: 'type' }, // status

                { name: 'created', type: 'date', dateFormat: 'c' },
                { name: 'updated', type: 'date', dateFormat: 'c' },

                { name: 'releaseRepositoryId' },
                { name: 'parentGroupId' },

                // exposed conditions (ie. properties)
                { name: 'userId', type: 'string' }, // this is 'owner'
                { name: 'ipAddress', type: 'string' },
                { name: 'userAgent', type: 'string' },

                { name: 'notifications', type: 'int' },
                { name: 'transitioning', type: 'boolean' },

                // optional list of additional properties
                {name: 'properties'}
            ],

            sortInfo: {
                field: 'repositoryId',
                direction: 'ASC'
            }
        });

        self.constructor.superclass.constructor.call(self, config);
    },

    /**
     * @public
     */
    nameForId: function(id) {
        var record = this.getById(id);
        if (record) {
            return record.get('name');
        }
        else {
            this.logDebug('Missing repository for ID:', id);
            // use id as name
            return id;
        }
    }
});