/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
NX.define('Sonatype.repoServer.ArtifactProcurementRulePopup', {
  extend : 'Ext.Window',
  constructor : function(node) {

    this.node = node;
    var disableDelete = !Sonatype.lib.Permissions.checkPermission('nexus:procurementresolution',
          Sonatype.lib.Permissions.DELETE);

    if (node.isRoot) {
      this.title = "everything (*:*:*)";
    }
    else if (node.attributes.type == 'G') {
      var groupId = node.attributes.path;
      this.title = groupId.substring(1, groupId.length - 1).replace(/\//g, '.');
    }
    else if (node.attributes.type == 'A') {
      this.title = node.attributes.groupId + ':' + node.attributes.artifactId;
    }
    else if (node.attributes.type == 'V' || node.attributes.type == 'artifact') {
      this.title = node.attributes.groupId + ':' + node.attributes.artifactId + ':' + node.attributes.version;
    }

    this.title = 'Rules for ' + this.title;

    this.treeRoot = new Ext.tree.TreeNode({
      text : 'root'
    });
    this.deletedNodes = [];

    Sonatype.repoServer.ArtifactProcurementRulePopup.superclass.constructor.call(this, {
      closable : true,
      autoWidth : false,
      width : 445,
      autoHeight : true,
      modal : true,
      constrain : true,
      resizable : false,
      items : [
        {
          xtype : 'form',
          frame : true,
          items : [
            {
              xtype : 'panel',
              layout : 'column',
              autoHeight : true,
              style : 'padding-left: 5px',
              items : [
                {
                  xtype : 'treepanel',
                  title : 'Applied Rules',
                  border : true,
                  bodyBorder : true,
                  bodyStyle : 'background-color: #FFFFFF; border: 1px solid #B5B8C8',
                  width : 300,
                  height : 300,
                  animate : true,
                  lines : false,
                  autoScroll : true,
                  containerScroll : true,
                  root : this.treeRoot,
                  rootVisible : false,
                  enableDD : false
                },
                {
                  xtype : 'panel',
                  width : 112,
                  items : [
                    {
                      xtype : 'button',
                      text : 'Remove',
                      style : 'padding-left: 6px',
                      minWidth : 105,
                      disabled : disableDelete,
                      handler : this.removeResolution,
                      scope : this
                    },
                    {
                      xtype : 'button',
                      text : 'Remove All',
                      style : 'padding-left: 6px; margin-top: 5px',
                      minWidth : 105,
                      disabled : disableDelete,
                      handler : this.removeAllResolutions,
                      scope : this
                    },
                    {
                      xtype : 'button',
                      text : 'Undo',
                      style : 'padding-left: 6px; margin-top: 5px',
                      minWidth : 105,
                      disabled : disableDelete,
                      handler : this.undoResolutions,
                      scope : this
                    }
                  ]
                }
              ]
            }
          ],
          buttons : [
            {
              text : 'Save',
              formBind : true,
              disabled : disableDelete,
              handler : this.saveResolutions,
              scope : this
            },
            {
              text : 'Cancel',
              formBind : false,
              handler : function(button, e) {
                this.close();
              },
              scope : this
            }
          ]
        }
      ]
    });

    this.loadResolutions();
  },

  loadResolutions : function() {
    var repoRec = this.node.ownerTree.payload;
    var appliedIds = this.node.attributes.appliedArtifactResolutionIds;
    var ids = "";
    var count = 0;
    if (appliedIds) {
      for (var i = 0; i < appliedIds.length; i++) {
        if (appliedIds[i]) {
          for (var j = 0; j < appliedIds[i].ruleIds.length; j++) {
            if (appliedIds[i].ruleIds[j]) {
              if (count > 0) {
                ids += ',';
              }
              ids += appliedIds[i].ruleIds[j];
              count++;
            }
          }
        }
      }
    }

    Ext.Ajax.request({
      method : 'GET',
      url : Nexus.procurement.configuration.PROCUREMENT_RESOLUTION_SERVICE_PATH + repoRec.data.id,
      params : {
        ids : ids
      },
      success : this.loadResolutionsHandler,
      scope : this
    });
  },
  getAppliedIconName : function(resolution) {
    var rule = resolution.ruleTypeId;
    var name = '';
    if (rule) {
      if (rule.toLowerCase() == 'simple') {
        if (resolution.properties && resolution.properties.length == 1) {
          var key = resolution.properties[0].key;
          var value = resolution.properties[0].value;
          if (key == "isApproved" && value == "false") {
            name = 'disapproved';
          }
          else if (key == "isApproved" && value == "true") {
            name = 'approved'
          }
          else {
            name = 'runtime';
          }
        }
        else {
          name = 'runtime'
        }
      }
      else {
        name = 'runtime';
      }
    }
    else {
      name = 'undefined';
    }
    return name;
  },

  loadResolutionsHandler : function(response, options) {
    new Ext.tree.TreeSorter(this.treeRoot.ownerTree, {
      property : 'resolutionId'
    })

    var rec = Ext.decode(response.responseText);
    if (rec.data) {
      for (var i = 0; i < rec.data.length; i++) {
        var resolution = rec.data[i];
        this.treeRoot.appendChild(new Ext.tree.TreeNode({
          text : this.getAppliedRuleText(resolution),
          resolutionId : resolution.id,
          leaf : true,
          icon : Sonatype.config.resourcePath + '/static/images/icons/procurement/'
                + (resolution.artifactCoordinate.artifactId == '*' ? 'folder-' : 'leaf-')
                + this.getAppliedIconName(resolution) + '.png'
        }));
      }
    }
    this.show();
  },

  getAppliedRuleText : function(rule) {
    var ruleText = rule.ruleTypeId;
    if (rule.ruleTypeId == "simple" && rule.properties && rule.properties.length == 1) {
      var key = rule.properties[0].key;
      var value = rule.properties[0].value;
      if (value == "true") {
        ruleText = "Approve for";
      }
      else {
        ruleText = "Deny for"
      }
    }
    else if (rule.ruleTypeId == "signature" && rule.properties && rule.properties.length == 1) {
      var key = rule.properties[0].key;
      var value = rule.properties[0].value;
      if (value == "true") {
        ruleText = "Signature must be present for";
      }
      else {
        ruleText = "Signature can be present/empty for";
      }
    }
    else {
      ruleText = rule.ruleTypeId
    }
    return (ruleText + " " + rule.artifactCoordinate.groupId + ":" + rule.artifactCoordinate.artifactId + ":"
          + rule.artifactCoordinate.version);
  },

  removeAllResolutions : function(button, e) {
    while (this.treeRoot.firstChild) {
      this.deletedNodes.push(this.treeRoot.firstChild);
      this.treeRoot.removeChild(this.treeRoot.firstChild);
    }
  },

  removeResolution : function(button, e) {
    var selectedNode = this.treeRoot.ownerTree.getSelectionModel().getSelectedNode();
    if (selectedNode) {
      this.deletedNodes.push(selectedNode);
      this.treeRoot.removeChild(selectedNode);
    }
  },

  saveResolutions : function(button, e) {
    if (this.deletedNodes.length) {
      var repoRec = this.node.ownerTree.payload;
      var repoName = repoRec.data.name;
      var ids = '';
      for (var i = 0; i < this.deletedNodes.length; i++) {
        if (i > 0) {
          ids += ',';
        }
        ids += this.deletedNodes[i].attributes.resolutionId;
      }

      Ext.Ajax.request({
        scope : this,
        method : 'DELETE',
        url : Nexus.procurement.configuration.PROCUREMENT_RESOLUTION_SERVICE_PATH + repoRec.data.id + '?ids=' + ids,
        callback : function(options, success, response) {
          if (success) {
            this.close();
            this.node.ownerTree.refreshNode(this.node);
          }
          else {
            Sonatype.utils.connectionError(response, 'Artifact resolution update failed in ' + repoName + '.');
          }
        }
      });
    }
    else {
      this.close();
    }
  },

  undoResolutions : function(button, e) {
    for (var i = 0; i < this.deletedNodes.length; i++) {
      this.treeRoot.appendChild(new Ext.tree.TreeNode(this.deletedNodes[i].attributes));
    }
    this.deletedNodes = [];
  }
});
