/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX,Ext,Sonatype,Nexus*/
NX.define('Sonatype.repoServer.LdapLoginTestWindow', {
  extend : 'Ext.Window',
  constructor : function(cfg) {
    var config = cfg || {};
    Ext.apply(this, config, {});

    Sonatype.repoServer.LdapLoginTestWindow.superclass.constructor.call(this, {
      title : 'Login Credentials',
      closable : true,
      autoWidth : false,
      width : 350,
      autoHeight : true,
      modal : true,
      constrain : true,
      resizable : false,
      draggable : false,
      items : [
        {
          xtype : 'form',
          labelWidth : 110,
          frame : true,
          monitorValid : true,
          items : [
            {
              xtype : 'panel',
              style : 'padding-left: 70px; padding-bottom: 10px',
              html : 'Enter credentials to test against the LDAP server.'
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Username',
              helpText : 'A username expected to be in the LDAP server.',
              anchor : Sonatype.view.FIELD_OFFSET,
              name : 'username',
              allowBlank : false,
              itemCls : 'required-field'
            },
            {
              xtype : 'textfield',
              inputType : 'password',
              fieldLabel : 'Password',
              helpText : 'A password used to login the user in LDAP server.',
              anchor : Sonatype.view.FIELD_OFFSET,
              name : 'password',
              allowBlank : false,
              itemCls : 'required-field'
            }
          ],
          buttons : [
            {
              text : 'Test',
              formBind : true,
              scope : this,
              handler : function() {
                this.testHandler();
              }
            },
            {
              text : 'Cancel',
              formBind : false,
              scope : this,
              handler : function() {
                this.cancelHandler();
              }
            }
          ]
        }
      ]
    });
  },

  testHandler : function() {
    // this is solely being used to build json from the form, not
    // doing the submit here
    var formSubmitter = new Ext.form.Action.sonatypeSubmit(this.formPanel.getForm(), {
          serviceDataObj : this.referenceData
        }),

        formJson = Ext.decode(formSubmitter.serializeForm(this.formPanel, this.formPanel.getForm())),

        requestJson = {
          data : {
            username : Sonatype.utils.base64.encode(this.find('name', 'username')[0].getValue()),
            password : Sonatype.utils.base64.encode(this.find('name', 'password')[0].getValue()),
            configuration : formJson.data
          }
        },
        params,
        isNew = this.ldapConfigurationContainer.isNew;

    if ( !isNew ) {
      params = {
        ldapServerId : this.ldapConfigurationContainer.record.get('id')
      };
    }

    Ext.Ajax.request({
      url : Sonatype.config.servicePath + '/ldap/logintest',
      method : 'PUT',
      params : params,
      jsonData : requestJson,
      scope : this,
      success : function(response, options) {
        Sonatype.MessageBox.alert('Check Login',
              'LDAP login completed successfully.').setIcon(Sonatype.MessageBox.INFO);
      },
      failure : function(response, options) {
        Sonatype.utils.connectionError(response, 'Error checking login.', false, {
          hideErrorStatus : true,
          decodeErrorResponse : true
        });
      }
    });
  },
  cancelHandler : function() {
    this.close();
  }
});