/*
 * Copyright (c) 2011-2013 Sonatype, Inc. All rights reserved.
 * Includes the third-party code listed at http://links.sonatype.com/products/rhc/pro/attributions.
 * "Sonatype" is a trademark of Sonatype, Inc.
 */
Sonatype.HealthCheckDetailWindow = function(config) {
    var config = config || {};
    var defaultConfig = {
        layout : 'fit',
        items : [ {
            xtype : 'iframepanel',
            loadMask : {
                msg : 'Loading Health Check Detail...'
            },
            autoShow : true,
            fitToParent : true,
            defaultSrc : function() {
                return config.url;
            }
        } ]
    };
    Ext.apply(this, config, defaultConfig);
    Sonatype.HealthCheckDetailWindow.superclass.constructor.call(this, {});
};

Ext.extend(Sonatype.HealthCheckDetailWindow, Ext.Panel, {});

HealthCheckProRepositoryContrib = function() {
    var me = this;
    Sonatype.Events.addListener('historyChanged', function(token) {
        if (token) {
            var toks = token.split(Sonatype.view.HISTORY_DELIMITER);
            if (toks[0] == 'healthCheckPro') {
                if (Sonatype.user.curr.isLoggedIn) {
                    if (me.hasHealthCheckDetailReadPerm()) {
                        var name = decodeURIComponent(toks[1]);
                        var repoId = decodeURIComponent(toks[2]);
                        var path = decodeURIComponent(toks[3]);

                        var id = me.toTabId(name, repoId, path);
                        var url = Sonatype.config.servicePath + '/healthCheckDetail/' + repoId + '/current/' + path;

                        Sonatype.view.mainTabPanel.addOrShowTab(id, Sonatype.HealthCheckDetailWindow, {
                            title : 'RHC-' + name,
                            url : url
                        });
                    }
                } else {
                    Sonatype.view.afterLoginToken = token;
                    Sonatype.repoServer.RepoServer.loginHandler();
                }
                //was handled by this event listener so return false to stop others
                return false;
            }
        }
    });

    Sonatype.Events.addListener('healthCheckWindowInit', function(repoContrib, toolbar, record, healthcheckdata) {
        // require user to be logged in before adding the button, also require that license supports details
        if (healthcheckdata.detailedReportSupported && me.hasHealthCheckDetailReadPerm() && Sonatype.user.curr.isLoggedIn) {
            if (Ext.isEmpty(healthcheckdata.healthCheckDetailUrl)) {
                toolbar.push('->', 'The next analysis will include the detailed report.');
            } else {
                toolbar.push('->', {
                    text : 'View Detailed Report',
                    handler : function(button) {
                        var url = healthcheckdata.healthCheckDetailUrl;
                        var repoId = healthcheckdata.id;
                        var name = record.get('name');
                        var versionBegin = url.indexOf('/' + healthcheckdata.id + '/') + ('/' + healthcheckdata.id + '/').length;
                        var versionEnd = url.indexOf('/', versionBegin);
                        var version = url.substring(versionBegin, versionEnd);
                        var path = url.substring(versionEnd + 1);

                        var id = me.toTabId(name, repoId, path);

                        Sonatype.view.mainTabPanel.addOrShowTab(id, Sonatype.HealthCheckDetailWindow, {
                            title : 'RHC-' + name,
                            url : url
                        });

                        repoContrib.closeHealthCheckWindow();
                    },
                    record : record
                });
            }
        }
    });

    me.hasHealthCheckDetailReadPerm = function() {
        return Sonatype.lib.Permissions.checkPermission('nexus:healthcheckdetail', Sonatype.lib.Permissions.READ);
    };

    me.toTabId = function(name, repoId, path) {
        return 'healthCheckPro' + Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(name) + Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(repoId)
                                + Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(path);
    };
};

Sonatype.HealthCheckProRepositoryContrib = new HealthCheckProRepositoryContrib();